package in.slanglabs.assistants.retail;

import android.app.Activity;
import android.app.Application;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;

import androidx.annotation.NonNull;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import in.slanglabs.assistants.base.AssistantUIPosition;
import in.slanglabs.assistants.retail.AssistantConfiguration;
import in.slanglabs.assistants.retail.AssistantError;
import in.slanglabs.assistants.retail.NavigationInfo;
import in.slanglabs.assistants.retail.NavigationUserJourney;
import in.slanglabs.assistants.retail.OrderInfo;
import in.slanglabs.assistants.retail.OrderManagementUserJourney;
import in.slanglabs.assistants.retail.SearchInfo;
import in.slanglabs.assistants.retail.SearchUserJourney;
import in.slanglabs.assistants.retail.SlangRetailAssistant;
import in.slanglabs.platform.SlangLocale;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;

/**
 * SlangRetailAssistantPlugin
 */
public class SlangRetailAssistantPlugin implements FlutterPlugin, MethodCallHandler, ActivityAware {

    private static final String TAG = "SlangRetailAssistant";

    private static final String CHANNEL_NAME = "plugins.flutter.io/slang_retail_assistant";
    private static final String CONFIG_ASSISTANT_ID = "assistantId";
    private static final String CONFIG_API_KEY = "apiKey";
    private static final String CONFIG_LOCALE = "defaultLocale";
    private static final String CONFIG_REQUESTED_LOCALES = "requestedLocales";
    private static final String CONFIG_TRIGGER_POSITION = "triggerPosition";
    private static final String CONFIG_CUSTOM_TRIGGER_ENABLE = "enableCustomTrigger";

    private MethodChannel channel;

    private Application application;
    private WeakReference<Activity> currentActivity;
    private SearchUserJourney searchUserJourney;

    /**
     * Default Constructor.
     *
     * <p>Use this constructor in an add to app scenario to gracefully handle activity and context
     * changes.
     */
    public SlangRetailAssistantPlugin() {
    }

    private SlangRetailAssistantPlugin(Application application) {
        this.application = application;
    }

    /**
     * Registers a plugin with the v1 embedding api {@code io.flutter.plugin.common}.
     *
     * <p>Calling this will register the plugin with the passed registrar. However plugins initialized
     * this way won't react to changes in activity or context, unlike {@link
     * SlangRetailAssistantPlugin}.
     */
    @SuppressWarnings("unused")
    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(registrar.messenger(), CHANNEL_NAME);
        channel.setMethodCallHandler(new SlangRetailAssistantPlugin(registrar.activity().getApplication()));
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPlugin.FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), CHANNEL_NAME);
        if (flutterPluginBinding.getApplicationContext() instanceof Application) {
            application = (Application) flutterPluginBinding.getApplicationContext();
        }
        channel.setMethodCallHandler(this);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "initialize":
                final Map<String, Object> arguments = call.arguments();
                SlangRetailAssistantUtils.setConfigOptions(arguments);
                try {

                    AssistantConfiguration config = new AssistantConfiguration.Builder()
                            .setAssistantId(SlangRetailAssistantUtils.getStringConfig(CONFIG_ASSISTANT_ID))
                            .setAPIKey(SlangRetailAssistantUtils.getStringConfig(CONFIG_API_KEY))
                            .setDefaultLocale(SlangRetailAssistantLocaleMap.getDefaultLocale(
                                    SlangRetailAssistantUtils.getStringConfig(
                                            CONFIG_LOCALE)))
                            .setRequestedLocales(
                                    SlangRetailAssistantLocaleMap.getRequestedLocales(
                                            SlangRetailAssistantUtils.getStringListConfig(
                                                    CONFIG_REQUESTED_LOCALES)))
                            .enableCustomTrigger(SlangRetailAssistantUtils
                                    .getBooleanConfig(CONFIG_CUSTOM_TRIGGER_ENABLE))
                            .setUIPosition(SlangRetailAssistantUtils
                                    .getAssistantUIConfig(CONFIG_TRIGGER_POSITION))
                            .build();

                    if (application != null) {
                        SlangRetailAssistant.initialize(application, config);
                    } else if (currentActivity != null && currentActivity.get() != null) {
                        SlangRetailAssistant.initialize(currentActivity.get().getApplication(), config);
                    } else {
                        Map<String, String> params = new HashMap<>();
                        params.put("type", "Invalid call to initialize");
                        channel.invokeMethod("onAssistantInitFailure", params);
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setAction":
                try {
                    SlangRetailAssistant.setAction(new SlangRetailAssistant.Action() {
                        @Override
                        public SearchUserJourney.AppState onSearch(final SearchInfo searchInfo,
                                                                   SearchUserJourney searchUserJourney) {
                            SlangRetailAssistantPlugin.this.searchUserJourney = searchUserJourney;
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onSearch",
                                                SlangRetailAssistantUtils.mapSearchInfo(searchInfo));
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                            return SearchUserJourney.AppState.WAITING;
                        }

                        @Override
                        public OrderManagementUserJourney.AppState onOrderManagement(
                                final OrderInfo orderInfo,
                                OrderManagementUserJourney orderManagementUserJourney) {
                            return OrderManagementUserJourney.AppState.UNSUPPORTED;
                        }

                        @Override
                        public NavigationUserJourney.AppState onNavigation(final NavigationInfo navigationInfo,
                                                                           NavigationUserJourney navigationUserJourney) {
                            return NavigationUserJourney.AppState.UNSUPPORTED;
                        }

                        @Override
                        public void onAssistantError(final AssistantError assistantError) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    Log.d(TAG, String.valueOf(assistantError));
                                    Map<String, Object> params = new HashMap<>();
                                    params.put("description", assistantError.getDescription());
                                    channel.invokeMethod("onAssistantError", params);
                                }
                            });
                        }
                    });
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }

                break;

            case "setLifecycleObserver":
                try {
                    SlangRetailAssistant.setLifecycleObserver(
                            new SlangRetailAssistant.LifecycleObserver() {

                        @Override
                        public void onAssistantInitSuccess() {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onAssistantInitSuccess", null);
                                    } catch (
                                            Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public void onAssistantInitFailure(final String description) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        Map<String, String> params = new HashMap<>();
                                        params.put("type", description);
                                        channel.invokeMethod("onAssistantInitFailure", params);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public void onAssistantInvoked() {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onAssistantInvoked", null);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public void onAssistantClosed(final boolean isCancelled) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        Map<String, Boolean> params = new HashMap<>();
                                        params.put("isCancelled", isCancelled);
                                        channel.invokeMethod("onAssistantClosed", params);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public void onAssistantLocaleChanged(final Locale changedLocale) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        Map<String, String> params = new HashMap<>();
                                        params.put("country", changedLocale.getCountry().toUpperCase());
                                        params.put("language", changedLocale.getLanguage());
                                        channel.invokeMethod("onAssistantLocaleChanged", params);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public boolean onUnrecognisedUtterance(final String utterance) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        Map<String, String> params = new HashMap<>();
                                        params.put("utterance", utterance);
                                        channel.invokeMethod("onUnrecognisedUtterance", params);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                            return false;
                        }

                        @Override
                        public void onUtteranceDetected(final String utterance) {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        Map<String, String> params = new HashMap<>();
                                        params.put("utterance", utterance);
                                        channel.invokeMethod("onUtteranceDetected", params);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public void onOnboardingSuccess() {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onOnboardingSuccess", null);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }

                        @Override
                        public void onOnboardingFailure() {
                            new Handler(Looper.getMainLooper()).post(new Runnable() {
                                @Override
                                public void run() {
                                    try {
                                        channel.invokeMethod("onOnboardingFailure", null);
                                    } catch (Exception exception) {
                                        sendAssistantError(exception);
                                    }
                                }
                            });
                        }
                    });
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }

                break;

            case "startConversation":
                String userJourney = call.argument("userJourney");
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.startConversation(
                                RetailUserJourney.fromString(userJourney),
                                currentActivity.get());
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "showTrigger":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().showTrigger(currentActivity.get());
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "hideTrigger":
                if (currentActivity != null && currentActivity.get() != null) {
                    try {
                        SlangRetailAssistant.getUI().hideTrigger(currentActivity.get());
                    } catch (Exception exception) {
                        sendAssistantError(exception);
                    }
                }
                break;

            case "setSearchSuccess":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setSuccess();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setSearchFailure":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setFailure();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setSearchItemNotFound":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setItemNotFound();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;


            case "setSearchItemOutOfStock":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setItemOutOfStock();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setSearchItemNotSpecified":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setItemNotSpecified();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setSearchNeedItemQuantity":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setNeedItemQuantity();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setSearchNeedDisambiguation":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setNeedDisambiguation();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setSearchStopDisambiguation":
                try {
                    if (searchUserJourney != null) {
                        searchUserJourney.setStopDisambiguation();
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "notifySearchActionCompleted":
                try {
                    String appStateString = call.argument("appState");
                    SearchUserJourney.AppState appState = SearchUserJourney.AppState.fromString(appStateString);
                    if (searchUserJourney != null) {
                        try {
                            searchUserJourney.notifyAppState(appState);
                        } catch (Exception exception) {
                            sendAssistantError(exception);
                        }
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setUserJourneyToContinue":
                try {
                    String currentUserJourney = call.argument("currentUserJourney");
                    String userJourneyToContinue = call.argument("userJourneyToContinue");
                    boolean isSpoken = call.argument("isSpoken");

                    Log.d(TAG, "setUserJourneyToContinue");
                    RetailUserJourney assistantUserJourney = RetailUserJourney.fromString(currentUserJourney);
                    switch (assistantUserJourney) {
                        case SEARCH:
                            if (searchUserJourney != null) {
                                searchUserJourney.setRetailJourneyToContinue(RetailUserJourney
                                        .fromString(userJourneyToContinue), isSpoken);
                            }
                            break;
                        default:
                            break;
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "clearUserJourneyToContinue":
                try {
                    String currentUserJourney = call.argument("currentUserJourney");
                    Log.d(TAG, "clearUserJourneyToContinue");
                    RetailUserJourney assistantUserJourney = RetailUserJourney.fromString(currentUserJourney);
                    switch (assistantUserJourney) {
                        case SEARCH:
                            if (searchUserJourney != null) {
                                searchUserJourney.clearRetailJourneyToContinue();
                            }
                            break;
                        default:
                            break;
                    }
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setAppDefaultUserJourney":
                try {
                    String defaultUserJourney = call.argument("userJourney");
                    SlangRetailAssistant.setAppDefaultUserJourney(RetailUserJourney.fromString(defaultUserJourney));
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "setAppDefaultSubDomain":
                try {
                    String appSubDomain = call.argument("appSubDomain");
                    SlangRetailAssistant.setAppDefaultSubDomain(AssistantSubDomain.fromString(appSubDomain));
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "clearSearchUserJourneyContext":
                try {
                    SearchUserJourney.getContext().clear();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;

            case "disableSearchUserJourneyContext":
                try {
                    SearchUserJourney.disablePreserveContext();
                } catch (Exception exception) {
                    sendAssistantError(exception);
                }
                break;
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPlugin.FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
    }

    @Override
    public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
        currentActivity = new WeakReference<>(binding.getActivity());
    }

    @Override
    public void onDetachedFromActivityForConfigChanges() {

    }

    @Override
    public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

    }

    @Override
    public void onDetachedFromActivity() {

    }

    private void sendAssistantError(final Exception exception) {
        new Handler(Looper.getMainLooper()).post(new Runnable() {
            @Override
            public void run() {
                Map<String, String> params = new HashMap<>();
                params.put("type", exception.getMessage());
                channel.invokeMethod("onAssistantError", params);
            }
        });
    }


}
